@@ -93,7 +93,7 @@ SEXP getEdgeIDs(SEXP x, int v, int alter, const char *neighborhood, int naOmit)
 /*Retrieve the IDs of all edges incident on v, in network x.  Outgoing or incoming edges are specified by neighborhood, while na.omit indicates whether or not missing edges should be omitted.  If alter>0, only edges whose alternate endpoints contain alter are returned.  The return value is a vector of edge IDs.*/
 {
   SEXP eids,newids,mel,ilist,olist,eplist;
-  int i,j,pc=0,ecount,*keep,dir;
+  int i,j,k,pc=0,ecount,*keep,dir;
   
   /*Enforce "combined" behavior unless x is directed*/
   dir=isDirected(x);
@@ -134,11 +134,33 @@ SEXP getEdgeIDs(SEXP x, int v, int alter, const char *neighborhood, int naOmit)
         PROTECT(eplist=vecAppend(ilist,olist)); pc++;
       }
       /*Check to see if any endpoint matches alter*/
+      /*Rprintf("\t\tchecking endpoints of EID %d\n",INTEGER(eids)[i]);*/
       keep[i]=0;
-      for(j=0;(j<length(eplist))&&(!keep[i]);j++)
-        if(INTEGER(eplist)[j]==alter){
-          keep[i]++;
+      if (dir | v!=alter){
+        /* does this still work in hypergraphic case?*/
+        for(j=0;(j<length(eplist))&&(!keep[i]);j++){
+          /*Rprintf("\t\t\tendpoint %d\n",INTEGER(eplist)[j]);*/
+          if(INTEGER(eplist)[j]==alter){
+            keep[i]++;
+          }
         }
+      } else { /* need to avoid false matches against v when checking alter */
+          for(j=0;(j<length(olist))&&(!keep[i]);j++){
+            /*Rprintf("\t\t\tendpoint j %d",INTEGER(olist)[j]);*/
+            if (INTEGER(olist)[j]==v){ /* if the out element == v */
+              /* scan the in list for the alter */
+              for(k=0;(k<length(ilist))&&(!keep[i]);k++){
+                /*Rprintf("\t\t\tendpoint k %d\n",INTEGER(ilist)[k]);*/
+                if(INTEGER(ilist)[k]==alter){
+                  keep[i]++;
+                }
+              }
+              
+            } 
+            
+        }
+        
+      }
     }
     if(naOmit){                      /*Remove missing edges*/
       /*Rprintf("\t\t\tEntering Omit step...\n");*/
@@ -151,9 +173,11 @@ SEXP getEdgeIDs(SEXP x, int v, int alter, const char *neighborhood, int naOmit)
   /*Rprintf("\tecount=%d\n",ecount);*/
   PROTECT(newids=allocVector(INTSXP,ecount)); pc++;  /*Create edge ID list*/
   ecount=0;
-  for(i=0;i<length(eids);i++)
-    if(keep[i])
+  for(i=0;i<length(eids);i++){
+    if(keep[i]){
       INTEGER(newids)[ecount++]=INTEGER(eids)[i];
+    }
+  }
   
  /* Rprintf("\tReturning %d edge IDs\n",length(newids));
   if(length(newids)>0)
@@ -506,6 +530,8 @@ SEXP setVertexAttribute(SEXP x, const char *attrname, SEXP value, int v)
 }
 
 
+
+
 /*R-CALLABLE ROUTINES--------------------------------------------------*/
 
 
@@ -514,6 +540,9 @@ SEXP addEdge_R(SEXP x, SEXP tail, SEXP head, SEXP namesEval, SEXP valsEval, SEXP
   int pc=0,i,j,mnext;
   SEXP el,atl,atlnam,navec,inl,outl,elnam,mel,newmel,oel,iel,ptr,elem,mnptr,gal;
   char buf[64];
+  
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++; 
 
   /*Make sure that we can read the head and tail lists*/
   PROTECT(inl = coerceVector(head, INTSXP)); pc++;
@@ -677,6 +706,9 @@ SEXP addEdges_R(SEXP x, SEXP tail, SEXP head, SEXP namesEval, SEXP valsEval, SEX
   SEXP el,atl,atlnam,navec,inl,outl,elnam,mel,newmel,oel,iel,ptr,elem,mnptr,gal;
   char buf[64];
   
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
+  
   /*Create enlarged master edge list*/
   PROTECT(mnptr=coerceVector(getListElement(getListElement(x,"gal"), "mnext"),INTSXP)); pc++;
   if(length(mnptr)==0)
@@ -863,6 +895,9 @@ SEXP addVertices_R(SEXP x, SEXP nv, SEXP vattr)
   int pc=0,n,ninc,i;
   SEXP ns,oel,iel,val,el,atts,newna;
   
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
+  
   /*Update the network size attribute*/
   PROTECT(nv=coerceVector(nv,INTSXP)); pc++;
   ninc=INTEGER(nv)[0];
@@ -936,6 +971,9 @@ SEXP deleteEdgeAttribute_R(SEXP x, SEXP attrname)
   int i,j,pc=0,n;
   SEXP anam,mel;
   
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
+  
   /*Remove the attributes....*/
   mel=getListElement(x,"mel");
   n=length(mel);
@@ -957,6 +995,9 @@ SEXP deleteEdges_R(SEXP x, SEXP eid)
 {
   int pc=0,i,j,e,opc;
   SEXP mel,el,head,tail,oel,iel,newvec;
+  
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
 
   /*Rprintf("deleteEdges; removing %d edges\n",length(eid));*/
   /*Coerce eid to the appropriate form*/
@@ -1012,6 +1053,9 @@ SEXP deleteNetworkAttribute_R(SEXP x, SEXP attrname)
   int i,pc=0;
   SEXP anam;
   
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
+  
   /*Remove the attributes....*/
   PROTECT(anam=coerceVector(attrname,STRSXP)); pc++;
   for(i=0;i<length(anam);i++){
@@ -1030,6 +1074,9 @@ SEXP deleteVertexAttribute_R(SEXP x, SEXP attrname)
   int i,j,pc=0,n;
   SEXP anam;
   
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
+  
   /*Remove the attributes....*/
   n=networkSize(x);
   PROTECT(anam=coerceVector(attrname,STRSXP)); pc++;
@@ -1051,6 +1098,9 @@ SEXP deleteVertices_R(SEXP x, SEXP vid)
   char neigh[]="combined";
   SEXP eids,nord,newsize,val,iel,oel;
   
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
+  
   /*Coerce vid to integer form, and remove any duplicates*/
   /*Rprintf("Size on entry: %d, length of vid:%d\n",networkSize(x),length(vid));*/
   PROTECT(vid=coerceVector(vecUnique(vid),INTSXP)); pc++;
@@ -1270,6 +1320,9 @@ SEXP permuteVertexIDs_R(SEXP x, SEXP vids)
   char neigh[] = "combined";
   SEXP eids,cvids,cpos,val,iel,oel,epl,mel,idlist,edge;
   PROTECT_INDEX ipx;
+  
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
 
   /*Set up the initial variables*/
   PROTECT(vids=coerceVector(vids,INTSXP)); pc++;
@@ -1340,6 +1393,9 @@ SEXP setEdgeAttribute_R(SEXP x, SEXP attrname, SEXP value, SEXP e)
   int i,pc=0;
   SEXP mel,el,atl;
   
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
+  
   /*Coerce edge IDs into integer format, and get the master edge list*/
   PROTECT(e=coerceVector(e,INTSXP)); pc++;
   mel=getListElement(x,"mel");
@@ -1368,6 +1424,52 @@ SEXP setEdgeAttribute_R(SEXP x, SEXP attrname, SEXP value, SEXP e)
   return x;
 }
 
+/* this version is essentially the same as above, but allows passing in lists of attrnames and lists of lists of values */
+SEXP setEdgeAttributes_R(SEXP x, SEXP attrnames, SEXP values, SEXP e)
+/*Sets the attribute in attrname to the values in (list) value, for each corresponding edge with respective ID in vector e.  If an edge referred to in e does not actually exist (i.e., its entry in mel is NULL), then that edge (and the associated value) is silently skipped.  Note that existing attribute entries are overwritten by this routine, where present...if the named attribute does not exist, a new entry is created for each edge in question.*/
+{
+  int i,j,pc=0;
+  SEXP mel,el,atl, value;
+  const char * name;
+  
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
+  
+  /*Coerce edge IDs into integer format, and get the master edge list*/
+  PROTECT(e=coerceVector(e,INTSXP)); pc++;
+  mel=getListElement(x,"mel");
+  
+  PROTECT(attrnames = coerceVector(attrnames,STRSXP)); pc++; /*Coerce to str*/
+  
+  /* for each attername, loop over the edges */
+  for(j=0;j<length(attrnames);j++){
+  /*For each edge ID in e, set the appropriate attribute to the
+  indicated value*/
+  PROTECT(value=VECTOR_ELT(values,j));pc++;
+  name=CHAR(STRING_ELT(attrnames,j));
+    for(i=0;i<length(e);i++){
+      el=VECTOR_ELT(mel,INTEGER(e)[i]-1);   /*Get the edge*/
+      if(el!=R_NilValue){                   /*Only proceed if edge exists*/
+        atl=getListElement(el,"atl");         /*Get attr list*/
+        PROTECT(atl=setListElement(atl,name, VECTOR_ELT(value,i)));                      /*Add/alter attribute*/
+        /*Note: b/c atl might have been extended, we have to re-write it into*/
+        /*the edge object.  The same is not true for the edge itself, b/c we*/
+        /*know that every edge must contain the element "atl"; the return*/
+        /*value is guaranteed to be the same as the original value, and we're*/
+        /*safe.  If setListElement ever changes this behavior, the following*/
+        /*line will no longer work as-is, and an additional line resetting el*/
+        /*within mel will be required!  You have been warned!*/
+        el=setListElement(el,"atl",atl);
+        UNPROTECT(1);
+      }
+    }
+  }
+    
+  /*Unprotect and return*/
+  UNPROTECT(pc);
+  return x;
+}
+
 
 SEXP setEdgeValue_R(SEXP x, SEXP attrname, SEXP value, SEXP e)
 /*Set attribute attrname on the edges of x indexed by e, using the values in (adjacency matrix) value.  Loops and multiplex edges are allowed here, but hypergraphs are not!*/
@@ -1376,6 +1478,9 @@ SEXP setEdgeValue_R(SEXP x, SEXP attrname, SEXP value, SEXP e)
   const char *anam;
   SEXP mel,el,atl,newval=R_NilValue;
   
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
+  
   /*Set things up*/
   mel=getListElement(x,"mel");
   PROTECT(e=coerceVector(e,INTSXP)); pc++;
@@ -1439,6 +1544,9 @@ SEXP setNetworkAttribute_R(SEXP x, SEXP attrname, SEXP value)
 {
   int i,pc=0;
   
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
+  
   /*Coerce the attribute names*/
   PROTECT(attrname=coerceVector(attrname,STRSXP)); pc++;
 
@@ -1457,6 +1565,9 @@ SEXP setVertexAttribute_R(SEXP x, SEXP attrname, SEXP value, SEXP v)
   int i,pc=0;
   SEXP val,vl;
   
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
+  
   /*Coerce vertex IDs into integer format, and get the vertex attribute list*/
   PROTECT(v=coerceVector(v,INTSXP)); pc++;
   val=getListElement(x,"val");
@@ -1473,4 +1584,38 @@ SEXP setVertexAttribute_R(SEXP x, SEXP attrname, SEXP value, SEXP v)
   return x;
 }
 
+SEXP setVertexAttributes_R(SEXP x, SEXP attrnames, SEXP values, SEXP v)
+/*sets the attribute in attrname to the values in (list) value, for each corresponding vertex with ID in vector v.  Existing attribute entries are overwritten by this routine, where present; new attributes are added, if they do not exist.*/
+{
+  int i,j,pc=0;
+  SEXP val,vl,value;
+  const char * name;
+  
+  /* force a copy of x to avoid lazy evaluation problems*/
+  PROTECT(x = duplicate(x)); pc++;
+  PROTECT(values); pc++;
+  
+  /*Coerce vertex IDs into integer format, and get the vertex attribute list*/
+  PROTECT(v=coerceVector(v,INTSXP)); pc++;
+  PROTECT(val=getListElement(x,"val"));pc++;
+  
+  PROTECT(attrnames = coerceVector(attrnames,STRSXP)); pc++; /*Coerce to str*/
+
+  /*Update the attribute lists*/
+  for(j=0;j<length(attrnames);j++){ /* loop over attributes to be set */
+    PROTECT(value=VECTOR_ELT(values,j));pc++;
+    name=CHAR(STRING_ELT(attrnames,j));
+    for(i=0;i<length(v);i++){ /* loop over vertics */
+      PROTECT(vl=setListElement(VECTOR_ELT(val,INTEGER(v)[i]-1), name,VECTOR_ELT(value,i))); /*Protect temporarily*/
+      SET_VECTOR_ELT(val,INTEGER(v)[i]-1,vl);
+      UNPROTECT(1);  /*Remove protection to avoid stack overflow*/
+    }
+  }
+
+  /*Unprotect and return*/
+  UNPROTECT(pc);
+  return x;
+}
+
+
 
